#
# SchoolTool - common information systems platform for school administration
# Copyright (c) 2007 Shuttleworth Foundation
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
"""
Lyceum journal interfaces.
"""
import zope.schema
from zope.interface import Interface
from zope.interface import Attribute
from zope.location.interfaces import ILocation

from schooltool.requirement.interfaces import IScoreSystem

from schooltool.lyceum.journal import LyceumMessage as _


class IEvaluateRequirement(Interface):

    def evaluate(person, requirement, grade, evaluator=None, score_system=None):
        """Add evaluation of a requirement."""

    def getEvaluation(person, requirement, default=None):
        """Get evaluation of a requirement."""


class ISectionJournalData(IEvaluateRequirement):
    """A journal for a section."""

    section = Attribute("""Section this data belongs to.""")

    def setGrade(person, meeting, grade):
        """Set a grade for a person participating in this meeting."""

    def getGrade(person, meeting, default=None):
        """Retrieve a grade for a person and a meeting."""

    def setAbsence(person, meeting, explained=True, value=None):
        """Mark an absence as an explained or unexplained one."""

    def getAbsence(person, meeting, default=False):
        """Retrieve the status of an absence."""

    def isAbsent(person, meeting):
        """Retrieve the status of an absence."""

    def isTardy(person, meeting):
        """Retrieve the status of an absence."""

    def gradedMeetings(person):
        """Returns a list of (meeting, grades) for a person."""

    def absentMeetings(person):
        """Returns a list of (meeting, absence) for a person."""


class ISectionJournal(ILocation):

    section = Attribute("Section this journal belongs to.")
    members = Attribute("List of students that belong to this section or any of the adjacent.")

    def setGrade(person, meeting, grade):
        """Set a grade for a person participating in this meeting."""

    def getGrade(person, meeting, default=None):
        """Retrieve a grade for a person and a meeting."""

    def setAbsence(person, meeting, explained=True):
        """Mark an absence as an explained or unexplained one."""

    def getAbsence(person, meeting, default=False):
        """Retrieve the status of an absence."""

    def isAbsent(person, meeting):
        """Retrieve the status of an absence."""

    def isTardy(person, meeting):
        """Retrieve the status of an absence."""

    def meetings():
        """List all possible meetings for this section."""

    def gradedMeetings(person):
        """Returns a list of (meeting, grades) for a person."""

    def absentMeetings(person):
        """Returns a list of (meeting, absence) for a person."""

    def hasMeeting(person, meeting):
        """Returns true if person should participate in a given meeting."""

    def findMeeting(meeting_id):
        """Returns the meeting object for this meeting id.

        The meeting might belong to any of the adjacent sections so it
        goes through all their calendars to find the meeting.
        """


class IAttendanceScoreSystem(IScoreSystem):

    scores = zope.schema.List(
        title=u'All values',
        description=u'(score, label)',
        value_type=zope.schema.Tuple(),
        required=True)

    tag_absent = zope.schema.List(
        title=u'Absence scores',
        value_type=zope.schema.TextLine(),
        required=True)

    tag_tardy = zope.schema.List(
        title=u'Tardy scores',
        value_type=zope.schema.TextLine(),
        required=True)

    tag_excused = zope.schema.List(
        title=u'Excused scores',
        value_type=zope.schema.TextLine(),
        required=True)

    tag_skip = zope.schema.List(
        title=u'Skip scores',
        value_type=zope.schema.TextLine(),
        required=True)

    def isAbsent(score):
        pass

    def isTardy(score):
        pass

    def isExcused(score):
        pass

    def isSkipped(score):
        pass


class IPersistentAttendanceScoreSystem(IAttendanceScoreSystem):

    hidden = zope.schema.Bool(
        title=u"Hidden Score System",
        required=False
        )


class IJournalScoreSystemPreferences(Interface):

    grading_scoresystem = zope.schema.Choice(
        title=_("Grading score system"),
        vocabulary="schooltool.lyceum.journal-grading-scoresystems",
        required=True)

    attendance_scoresystem = zope.schema.Choice(
        title=_("Attendance score system"),
        vocabulary="schooltool.lyceum.journal-attendance-scoresystems",
        required=True)

    student_scoresystem = zope.schema.Choice(
        title=_("Student Attendance"),
        vocabulary="schooltool.lyceum.journal-attendance-scoresystems",
        required=True)

    teacher_scoresystem = zope.schema.Choice(
        title=_("Teacher Attendance"),
        vocabulary="schooltool.lyceum.journal-attendance-scoresystems",
        required=True)


class IAvailableScoreSystems(Interface):
    """A marker interface to get a list of available scoresystems."""


class ITeacherAbsences(Interface):

    absences = zope.schema.Dict(
        title=u'Teacher Absences',
        key_type=zope.schema.Date(title=u'Date'),
        value_type=zope.schema.Tuple(title=u'(State ID, Substitute username)'))


class Time(zope.schema.Time):

    pass


class ITimestamp(Interface):

    arrived = Time(
        title=_('Arrived'),
        description=_('HH:MM'),
        required=False)

    left = Time(
        title=_('Left'),
        description=_('HH:MM'),
        required=False)

    returned = Time(
        title=_('Returned'),
        description=_('HH:MM'),
        required=False)


class IAttendanceOptions(Interface):

    time_enabled = zope.schema.Bool(
        title=_('Use time options'),
        required=False)

    students_am_enabled = zope.schema.Bool(
        title=_('AM'),
        default=True,
        required=False)

    students_am_start = Time(
        title=_('AM start time'),
        description=_('HH:MM'),
        required=False)

    students_am_end = Time(
        title=_('AM end time'),
        description=_('HH:MM'),
        required=False)

    students_pm_enabled = zope.schema.Bool(
        title=_('PM'),
        default=True,
        required=False)

    students_pm_start = Time(
        title=_('PM start time'),
        description=_('HH:MM'),
        required=False)

    students_pm_end = Time(
        title=_('PM end time'),
        description=_('HH:MM'),
        required=False)

    teachers_am_enabled = zope.schema.Bool(
        title=_('AM'),
        default=True,
        required=False)

    teachers_am_start = Time(
        title=_('AM start time'),
        description=_('HH:MM'),
        required=False)

    teachers_am_end = Time(
        title=_('AM end time'),
        description=_('HH:MM'),
        required=False)

    teachers_pm_enabled = zope.schema.Bool(
        title=_('PM'),
        default=False,
        required=False)

    teachers_pm_start = Time(
        title=_('PM start time'),
        description=_('HH:MM'),
        required=False)

    teachers_pm_end = Time(
        title=_('PM end time'),
        description=_('HH:MM'),
        required=False)

    students_hours = zope.schema.Float(
        title=_('Students number of hours'),
        default=7.0,
        required=False)

    students_timestamp_codes = zope.schema.List(
        title=_('Students timestamp trigger codes'),
        value_type=zope.schema.Choice(
            title=u'Attendance code',
            source='schooltool.lyceum.journal.students_attendance_codes'),
        required=False)

    teachers_hours = zope.schema.Float(
        title=_('Teachers number of hours'),
        default=7.0,
        required=False)

    teachers_timestamp_codes = zope.schema.List(
        title=_('Teachers timestamp trigger codes'),
        value_type=zope.schema.Choice(
            title=u'Attendance code',
            source='schooltool.lyceum.journal.teachers_attendance_codes'),
        required=False)
